			include "propeller.i"
			include "demo.i"

			org	0

			jmp	#unpack
			byte	XTAL1 + PLL16X
			byte	0			' checksum
			word	@progbase		' program base
nextbyte		word	@progend		' variable base
			word	@progend + 8		' stack base
gc_count		word	@spincode		' spin entry point
			word	@progend + 12		' stack end
progbase
gc_result		word	@progend - @progbase	' object size
			byte	2, 0
up_count		word	@spincode - @progbase	' pointer to first method
			word	0			' number of variables

t1
spincode		byte	0x35, 0x35, 0x35, 0x2c

	' Unpack is a normal subroutine, but initially its ret instruction
	' (which would usually be a jmp #0) is preprogrammed to jump to stageone.
	' That's why we may call it using jmp #unpack.

	' Packed data is stored in a bitfile.
	' First, the number of items (stored as a chunk).
	' Then, a sequence of items, where each item is one of:
	'    0 nnnn ...         read (n + 1) bytes from the bitstream
	'    1 oooooooo nnnnnn  copy (n + 2) bytes from (current position - 1 - o)

	' Input:
	'    up_dest   Start of unpack area - 1
	'    reader    Initial bitfile reader state, i.e. address of packed data
	' Output:
	'    up_dest   Final byte of unpacked data
	'    reader    Past final bit of packed data

unpack
			mov	gc_count, #COUNTBITS
			call	#getchunk
			mov	up_count, gc_result
up_loop	
p_unpack		call	#getbit			' This call is later modified to call the playroutine
p_unpack_ret	if_nc	jmp	#:read

			mov	gc_count, #OFFSETBITS
			call	#getchunk
			mov	t1, up_dest
			sub	t1, gc_result
			mov	gc_count, #COPYBITS
			call	#getchunk
			add	gc_result, #2
:copyloop		rdbyte	PAR, t1			' Use PAR shadow RAM as temporary storage
			add	t1, #1
			add	up_dest, #1
			wrbyte	PAR, up_dest
			djnz	gc_result, #:copyloop
			jmp	#:tail
:read
			mov	gc_count, #READBITS
			call	#getchunk
			mov	t1, gc_result
			add	t1, #1
:readloop		mov	gc_count, #8
			call	#getchunk
			add	up_dest, #1
			wrbyte	gc_result, up_dest
			djnz	t1, #:readloop
:tail			djnz	up_count, #up_loop

unpack_ret		jmp	#stageone

	' Pre-initialized variables.

reader			long	@bf_init
up_dest			long	(@stageone - 1) | (@filetable << 16)	' Upper 16 bits are ignored by the unpacker.

	' A bitfile is stored in consecutive bytes (lsb to msb within each byte) at increasing addresses.
	' The bitfile reader state is kept in a single register:
	'    00000000_?0000000_aaaaaaaa_aaaaaaaa (identified by zero high-byte)
	'    a = address of next byte, next byte will be read, lsb returned, a incremented, next state:
	'    01000000_0aaaaaaa_aaaaaaaa_abbbbbbb
	'    a = address of next byte, b = enqueued bits, lsb of b will be returned, next state:
	'    00100000_00aaaaaa_aaaaaaaa_aabbbbbb
	'    ...
	'    00000000_10000000_aaaaaaaa_aaaaaaaa
	'    all bits have been shifted out, and we have returned to the initial state

getbyte			rdbyte	nextbyte, reader

getbit_back		movi	nextbyte, #$100
			add	reader, #1
			shl	reader, #8
			or	reader, nextbyte
getbit
			test	reader, hff000000	wz
		if_nz	shr	reader, #1		wc
getbit_ret	if_nz	ret

p_getbit		jmp	#getbyte		' This call is later modified to read from EEPROM

hff000000		long	$ff000000

	' Read a chunk of gc_count bits from the current bitfile.
	' Chunks are stored with msb first.

getchunk
			mov	gc_result, #0
:lp			call	#getbit
			rcl	gc_result, #1
			djnz	gc_count, #:lp
getchunk_ret		ret

			fit	UNPACK_LIBREGS

stageone
			' This is the entry point after unpacking init.

			' First time we're here, there's just a coginit instruction which will restart cog 0 with the init program.

			' The init program will unpack init again (which is kind of useless, but it saves an instruction in the
			' boot loader, and it will make sure that the bitstream reader is positioned at the start of the packed
			' vga program). Then it will go on to do all sorts of initialization. Finally, it will restart cog 0 with
			' the play program. Once more, init will be unpacked before we get there, and this is a dummy operation.

			coginit	OUTB		' Replaces cog 0 (OUTB == 0)

' Just a little note to encourage reverse engineering.

banner			byte	"[ quaerendo invenietis ]"

' The music data is accessed directly from EEPROM, so the corresponding hub RAM
' area (up to $0800) will be put to use immediately as a temporary unpacking area.

			include	"song.s"
			include	"song_h.s"

' The following packed file will be unpacked from RAM, and must be somewhere in the
' range [$0800, $73ff]. The song data is big enough to guarantee the former, and
' an hfit will verify the latter.

			align	4
bf_init			file	"init_vga_synth_play.pkg"

			hfit	$7400

' The following packed files will be unpacked directly from EEPROM.

bf_viewdither		file	"viewdither.pkg"
bf_mandelbrot		file	"mandelbrot.pkg"
bf_ztunnel		file	"ztunnel.pkg"
bf_raster		file	"raster.pkg"
bf_tentacles		file	"tentacles.pkg"
bf_smoke		file	"smoke.pkg"
bf_viewsmoke		file	"viewsmoke.pkg"
bf_voxelscape		file	"voxelscape.pkg"
bf_viewzoom		file	"viewzoom.pkg"
bf_mist			file	"mist.pkg"
bf_cube			file	"cube.pkg"
bf_sky			file	"sky.pkg"
bf_spinner		file	"spinner.pkg"
bf_prevoxel		file	"prevoxel.pkg"
bf_viewtree		file	"viewtree.pkg"
bf_viewtwirl		file	"viewtwirl.pkg"
bf_viewbinary		file	"viewbinary.pkg"
bf_viewcubes		file	"viewcubes.pkg"
bf_viewgreets		file	"viewgreets.pkg"
bf_viewcircles		file	"viewcircles.pkg"
bf_viewend		file	"viewend.pkg"
bf_viewscroll		file	"viewscroll.pkg"
bf_viewlogo_tu		file	"viewlogo_turbulence.pkg"
bf_viewpr_lft		file	"viewpresents_lftlogo.pkg"
bf_flower_vf		file	"flower_viewflower.pkg"
bf_zoompic_gr		file	"zoompic_greets.pkg"
bf_blob			file	"blob.pkg"
bf_tree			file	"tree.pkg"
bf_twirl		file	"twirl.pkg"
bf_scrolltext		file	"scrolltext.pkg"

bf_script		file	"script.bin"
bf_script_h		file	"script_h.bin"

			' The file table is accessed from EEPROM.

			align	4

filetable		word	@bf_song
			word	@bf_tracks
			word	@bf_instruments
			word	@bf_script
			word	@bf_raster
			word	@bf_viewdither
			word	@bf_ztunnel
			word	@bf_blob
			word	@bf_tentacles
			word	@bf_smoke
			word	@bf_viewsmoke
			word	@bf_mist
			word	@bf_voxelscape
			word	@bf_viewzoom
			word	@bf_zoompic_gr
			word	@bf_mandelbrot
			word	@bf_viewpr_lft
			word	@bf_cube
			word	@bf_sky
			word	@bf_viewtree
			word	@bf_tree
			word	@bf_viewtwirl
			word	@bf_twirl
			word	@bf_viewbinary
			word	@bf_viewcubes
			word	@bf_viewgreets
			word	@bf_prevoxel
			word	@bf_viewlogo_tu
			word	@bf_spinner
			word	@bf_viewcircles
			word	@bf_viewend
			word	@bf_flower_vf

			' 32 entries later...

			word	@bf_song_h
			word	@bf_tracks_h
			word	@bf_instruments_h
			word	@bf_script_h
			word	@bf_viewscroll
			word	@bf_cube
			word	@bf_viewdither
			word	@bf_raster
			word	@bf_scrolltext

			align	4
			vhfit	32768 - 32 - 64
			align	32768 - 32 - 64

			word	0
			byte	" turbulence ", 0, 0
			word	0
			byte	"  lft 2009  ", 0, 0
			long	0
progend
			long	$fff9ffff
			long	$fff9ffff
			long	0
			long	0[12]

