' Channel data struct for the music sequencer.

CH_WAVE		=	0			' 0mmmmmmmm_00000_0aaaaaaaa_0vvvvvvvv
CH_TRACK	=	4			' bitstream
CH_INSTR	=	8			' bitstream

CH_FREQ		=	12			' unsigned word
CH_INOTE	=	14			' unsigned word, fixed point 8.8

CH_TIMER	=	16			' unsigned byte
CH_SLIDE	=	17			' signed byte
CH_FADE		=	18			' signed byte
CH_SWEEP	=	19			' unsigned byte

CH_LASTINSTR	=	20			' unsigned byte
CH_TNOTE	=	21			' unsigned byte
CH_VRATE	=	22			' unsigned byte
CH_VDEPTH	=	23			' unsigned byte

CH_TRANSP	=	24			' signed byte
CH_LASTT	=	25			' unsigned byte
CH_VPOS		=	26			' unsigned byte

CH_SIZEOF	=	28


SAMPLEFREQ	=	28000
TEMPO		=	4
TRACKLEN	=	32
TRACKBITS	=	7
TRANSPBITS	=	5
NOTEBITS	=	7
INSTRBITS	=	5

COUNTBITS	=	11
READBITS	=	4
COPYBITS	=	6
OFFSETBITS	=	8

' rast_cmd

RAST_CLEAR	=	1
RAST_TRIANGLE	=	2
RAST_QUAD	=	3
RAST_GLOW	=	4
RAST_BLENDTEX	=	5
RAST_BLENDTRI	=	6

' ditherstyle

DITHER_2_6	=	0
DITHER_8	=	1
DITHER_SKY	=	2
DITHER_VOXEL	=	3
DITHER_MANDEL	=	4
DITHER_HALF	=	5

UNPACK_LIBREGS	=	53		' Number of registers reserved by unpack.i
EEPROM_LIBREGS	=	59		' Number of registers reserved by eeprom.i

zero		=	OUTB		' No need to define our own zero register when OUTB will do fine

' Hub RAM addresses that are valid during boot, init and play setup.

		horg	0

		hres	4 * 37 + 2	' Offset to upper half of initial up_dest
filetableptr	hres	2

' Bitfiles.

BF_SONG		=	0
BF_TRACKS	=	1
BF_INSTRUMENTS	=	2
BF_SCRIPT	=	3

BF_RASTER	=	4
BF_VIEWDITHER	=	5
BF_ZTUNNEL	=	6
BF_BLOB		=	7
BF_TENTACLES	=	8
BF_SMOKE	=	9
BF_VIEWSMOKE	=	10
BF_MIST		=	11
BF_VOXELSCAPE	=	12
BF_VIEWZOOM	=	13
BF_ZOOMPIC_GR	=	14
BF_MANDELBROT	=	15
BF_VIEWPR_LFT	=	16
BF_CUBE		=	17
BF_SKY		=	18
BF_VIEWTREE	=	19
BF_TREE		=	20
BF_VIEWTWIRL	=	21
BF_TWIRL	=	22
BF_VIEWBINARY	=	23
BF_VIEWCUBES	=	24
BF_VIEWGREETS	=	25
BF_PREVOXEL	=	26
BF_VIEWLOGO_TU	=	27
BF_SPINNER	=	28
BF_VIEWCIRCLES	=	29
BF_VIEWEND	=	30
BF_FLOWER_VF	=	31

BF_H_VIEWSCROLL	=	4
BF_H_CUBE	=	5
BF_H_VIEWDITHER	=	6
BF_H_RASTER	=	7
BF_H_SCROLLTEXT	=	8

' Hub RAM addresses that are valid while the effects are running.

		horg	0

graphics0	hres	128*116
graphics1	hres	128*116

unpack_area	hres	2048

palette_area	hres	256

extra_area	hres	256

		hfit	$7e00

channels	hres	6*CH_SIZEOF

freqtable	hres	2*94		' must follow channels

nextvbl		hres	4

rast_stride				' A longword (16.16).
rast_x1		hres	2
rast_y1		hres	2
rast_dest				' A word (pointer).
rast_x2		hres	2
nextline				' A word. Protected by lock 0.
rast_y2		hres	2
rast_x3		hres	2
rast_y3		hres	2
rast_x4		hres	2
rast_y4		hres	2

' These will be accessed together as a longword.
backbuf		hres	2
frontbuf	hres	2

rastercmd	hres	1
slave_enable
rast_colour	hres	1
internalvbl	hres	1
ditherstyle	hres	1

		hfit	$7f80		' Remaining 128 bytes are used for audio visualization.

' Reduced graphics buffer for voxelscape.

		horg	0

perlin		hres	256
graphics0r	hres	128 * 115
graphics1r	hres	128 * 115

' Reduced graphics buffer for spinner.

		horg	0

		hres	45 * 128	' logo
graphics0h	hres	128 * 64
graphics1h	hres	128 * 64

' This file will be included by others. Leave the hub address as we found it.
		horg	0

