#include <stdint.h>

typedef uint8_t		u8;
typedef uint16_t	u16;
typedef uint32_t	u32;
typedef int8_t		s8;
typedef int16_t		s16;
typedef int32_t		s32;

#define MAXILEN 64
#define MAXSLEN 256
#define TRACKLEN 32
#define MAXINSTR 64
#define MAXTRACK 256

#define SAMPLEFREQ 28000

#define TEMPO 4

#define TRACKNUMBITS 7
#define TRANSPBITS 5
#define NOTEBITS 7
#define INSTRBITS 5
#define TRACKPTRBITS 16


struct instrument {
	u16		line[MAXILEN];
	int		length;
};

struct tline {
	u8		note;
	u8		instr;
};

struct track {
	struct tline	line[TRACKLEN];
};

struct sline {
	u8		track[8];
	u8		transp[8];
};

struct song {
	struct sline	line[MAXSLEN];
	int		length;
};

struct osc {
	u32	phase;
	u16	freq;
	u8	volume;
	u8	amount;
	u8	pwidth;
	u8	multiplier;
	u8	distort;
	u8	sine;
};

struct channel {
	u8	tnum;
	u8	transp;
	u8	lasti;

	u8	inum;
	u8	iptr;
	u8	iloop;
	u8	timer;
	u8	tnote;

	u16	inote;	// fixed point 8.8
	s8	bendd;
	s8	vold;
	u8	vibrpos;
	u8	vibrrate;
	u8	vibrdepth;

	s8	amountd;
	s8	pwm;
};

extern struct instrument instrument[MAXINSTR];
extern struct track track[MAXTRACK];
extern struct song song;

extern struct osc osc[8];
extern struct channel channel[8];
extern volatile int songpos, trackpos, tracktimer;
extern int playtr, playso;
extern int currch;

extern char *filename;

extern u16 sine[2048];
extern u16 freqtable[96];

void sample(s16 *left, s16 *right);
void playroutine();
u16 readinstr(u8 inum, u8 iptr);
void plonk(u8 cn, u8 note, u8 instr);
void guimain();
void iplonk(u8 cn, u8 note, u8 instr);
void silence();
void playsong(int pos);
void playtrack(int tr);
int loadfile();
int savefile();
void beep();
